@@ -14,13 +14,12 @@
 
 import logging
 import os
+import queue
 import threading
-import time
 import traceback
 
 from ovs.db import idl
 from ovs import poller
-from six.moves import queue as Queue
 
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import exceptions
@@ -33,7 +32,7 @@ else:
 LOG = logging.getLogger(__name__)
 
 
-class TransactionQueue(Queue.Queue, object):
+class TransactionQueue(queue.Queue, object):
     def __init__(self, *args, **kwargs):
         super(TransactionQueue, self).__init__(*args, **kwargs)
         self._wait_queue = connection_utils.WaitQueue(
@@ -42,7 +41,7 @@ class TransactionQueue(Queue.Queue, object):
     def get_nowait(self, *args, **kwargs):
         try:
             result = super(TransactionQueue, self).get_nowait(*args, **kwargs)
-        except Queue.Empty:
+        except queue.Empty:
             return None
         self._wait_queue.alert_notification_consume()
         return result
@@ -66,7 +65,7 @@ class Connection(object):
         """
         self.timeout = timeout
         self.txns = TransactionQueue(1)
-        self.lock = threading.RLock()
+        self.lock = threading.Lock()
         self.idl = idl
         self.thread = None
         self._is_running = None
@@ -76,9 +75,7 @@ class Connection(object):
         with self.lock:
             if self.thread is not None:
                 return False
-            if not self.idl.has_ever_connected() or self._is_running is False:
-                if self._is_running is False:  # stop() was called
-                    self.idl.force_reconnect()
+            if not self.idl.has_ever_connected():
                 idlutils.wait_for_change(self.idl, self.timeout)
                 try:
                     self.idl.post_connect()
@@ -96,44 +93,38 @@ class Connection(object):
         while self._is_running:
             # If we fail in an Idl call, we could have missed an update
             # from the server, leaving us out of sync with ovsdb-server.
-            # It is not safe to continue without restarting the connection.
-            # Though it is likely that the error is unrecoverable, keep trying
-            # indefinitely just in case.
+            # It is not safe to continue without restarting the connection,
+            # though it is likely that the error is unrecoverable, so only try
+            # a few times before bailing completely.
             try:
                 self.idl.wait(self.poller)
                 self.poller.fd_wait(self.txns.alert_fileno, poller.POLLIN)
+                # TODO(jlibosva): Remove next line once losing connection to
+                #                 ovsdb is solved.
+                self.poller.timer_wait(self.timeout * 1000)
                 self.poller.block()
-                with self.lock:
-                    self.idl.run()
+                self.idl.run()
             except Exception as e:
                 # This shouldn't happen, but is possible if there is a bug
                 # in python-ovs
                 errors += 1
                 LOG.exception(e)
-                with self.lock:
+                if errors <= 3:
                     self.idl.force_reconnect()
-                    try:
-                        idlutils.wait_for_change(self.idl, self.timeout)
-                    except Exception as e:
-                        # This could throw the same exception as idl.run()
-                        # or Exception("timeout"), either way continue
-                        LOG.exception(e)
-                sleep = min(2 ** errors, 60)
-                LOG.info("Trying to recover, sleeping %s seconds", sleep)
-                time.sleep(sleep)
-                continue
+                    idlutils.wait_for_change(self.idl, self.timeout)
+                    continue
+                self._is_running = False
+                break
             errors = 0
             txn = self.txns.get_nowait()
             if txn is not None:
                 try:
-                    with self.lock:
-                        txn.results.put(txn.do_commit())
+                    txn.results.put(txn.do_commit())
                 except Exception as ex:
                     er = idlutils.ExceptionResult(ex=ex,
                                                   tb=traceback.format_exc())
                     txn.results.put(er)
                 self.txns.task_done()
-        self.idl.close()
 
     def stop(self, timeout=None):
         if not self._is_running:
@@ -151,7 +142,7 @@ class Connection(object):
         # run when we are started
         try:
             self.txns.put(txn, timeout=self.timeout)
-        except Queue.Full:
+        except queue.Full:
             raise exceptions.TimeoutException(commands=txn.commands,
                                               timeout=self.timeout)
 
