@@ -49,9 +49,9 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
         return self.db_add('Logical_Switch', switch_uuid, 'dns_records',
                            dns_uuid)
 
-    def ls_remove_dns_record(self, switch_uuid, dns_uuid):
+    def ls_remove_dns_record(self, switch_uuid, dns_uuid, if_exists=False):
         return self.db_remove('Logical_Switch', switch_uuid, 'dns_records',
-                              dns_uuid)
+                              dns_uuid, if_exists=if_exists)
 
     def acl_add(self, switch, direction, priority, match, action, log=False,
                 may_exist=False, **external_ids):
@@ -64,6 +64,35 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def acl_list(self, switch):
         return cmd.AclListCommand(self, switch)
 
+    def pg_acl_add(self, port_group, direction, priority, match, action,
+                   log=False, may_exist=False, **external_ids):
+        return cmd.PgAclAddCommand(self, port_group, direction, priority,
+                                   match, action, log, may_exist,
+                                   **external_ids)
+
+    def pg_acl_del(self, port_group, direction=None, priority=None,
+                   match=None):
+        return cmd.PgAclDelCommand(self, port_group, direction, priority,
+                                   match)
+
+    def pg_acl_list(self, port_group):
+        return cmd.PgAclListCommand(self, port_group)
+
+    def qos_add(self, switch, direction, priority, match, rate=None,
+                burst=None, dscp=None, may_exist=False, **columns):
+        return cmd.QoSAddCommand(self, switch, direction, priority, match,
+                                 rate, burst, dscp, may_exist, **columns)
+
+    def qos_del(self, switch, direction=None, priority=None, match=None):
+        return cmd.QoSDelCommand(self, switch, direction, priority, match)
+
+    def qos_list(self, switch):
+        return cmd.QoSListCommand(self, switch)
+
+    def qos_del_ext_ids(self, lswitch_name, external_ids, if_exists=True):
+        return cmd.QoSDelExtIdCommand(self, lswitch_name, external_ids,
+                                      if_exists=if_exists)
+
     def lsp_add(self, switch, port, parent_name=None, tag=None,
                 may_exist=False, **columns):
         return cmd.LspAddCommand(self, switch, port, parent_name, tag,
@@ -153,6 +182,12 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lrp_get_enabled(self, port):
         return cmd.LrpGetEnabledCommand(self, port)
 
+    def lrp_set_options(self, port, **options):
+        return cmd.LrpSetOptionsCommand(self, port, **options)
+
+    def lrp_get_options(self, port):
+        return cmd.LrpGetOptionsCommand(self, port)
+
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
         return cmd.LrRouteAddCommand(self, router, prefix, nexthop, port,
@@ -244,26 +279,43 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
             ips = " ".join(utils.normalize_ip_port(ip) for ip in ips)
         return self.db_add('DNS', uuid, 'records', {hostname: ips})
 
-    def dns_remove_record(self, uuid, hostname):
-        return self.db_remove('DNS', uuid, 'records', hostname)
+    def dns_remove_record(self, uuid, hostname, if_exists=False):
+        return self.db_remove('DNS', uuid, 'records', hostname,
+                              if_exists=if_exists)
 
     def dns_set_external_ids(self, uuid, **external_ids):
         return cmd.DnsSetExternalIdsCommand(self, uuid, **external_ids)
 
-    def pg_add(self, name, may_exist=False, **columns):
+    def pg_add(self, name=None, may_exist=False, **columns):
         return cmd.PgAddCommand(self, name, may_exist=may_exist, **columns)
 
     def pg_del(self, name, if_exists=False):
         return cmd.PgDelCommand(self, name, if_exists=if_exists)
 
     def pg_add_ports(self, pg_id, lsp):
-        return cmd.PgAddDataCommand(self, pg_id, lsp=lsp)
+        return cmd.PgAddPortCommand(self, pg_id, lsp=lsp)
 
     def pg_del_ports(self, pg_id, lsp, if_exists=False):
-        return cmd.PgDelDataCommand(self, pg_id, lsp=lsp, if_exists=if_exists)
+        return cmd.PgDelPortCommand(self, pg_id, lsp=lsp, if_exists=if_exists)
 
-    def pg_add_acls(self, pg_id, acl):
-        return cmd.PgAddDataCommand(self, pg_id, acl=acl)
+    def pg_get(self, pg):
+        return cmd.PgGetCommand(self, pg)
 
-    def pg_del_acls(self, pg_id, acl, if_exists=False):
-        return cmd.PgDelDataCommand(self, pg_id, acl=acl, if_exists=if_exists)
+    def ha_chassis_group_add(self, name, may_exist=False, **columns):
+        return cmd.HAChassisGroupAddCommand(
+            self, name, may_exist=may_exist, **columns)
+
+    def ha_chassis_group_del(self, name, if_exists=False):
+        return cmd.HAChassisGroupDelCommand(self, name, if_exists=if_exists)
+
+    def ha_chassis_group_get(self, name):
+        return cmd.HAChassisGroupGetCommand(self, name)
+
+    def ha_chassis_group_add_chassis(self, hcg_id, chassis, priority,
+                                     **columns):
+        return cmd.HAChassisGroupAddChassisCommand(
+            self, hcg_id, chassis, priority, **columns)
+
+    def ha_chassis_group_del_chassis(self, hcg_id, chassis, if_exists=False):
+        return cmd.HAChassisGroupDelChassisCommand(
+            self, hcg_id, chassis, if_exists=if_exists)
